#!/usr/bin/perl -w
#
# $Id: Connection.pm,v 1.8 2003/06/11 06:59:05 solovam Exp $
#
# Connection class (a connection to Kismet server)
#
package Connection;

use IO::Socket;
use IO::Select;
use strict;

my %network_type = (network_ap => 0, network_adhoc => 1, network_probe => 2, network_turbocell => 3, network_data => 4, network_remove => 5);
my %address_type = (address_none => 0, address_factory => 1, address_udp => 2, address_arp => 3, address_tcp => 4, address_dhcp => 5, address_group => 6);
my %carrier_type = (carrier_unknown => 0, carrier_80211b => 1, carrier_80211a => 2, carrier_80211g => 3, carrier_80211 => 4);
my %client_type = (client_unknown => 0, client_fromds => 1, client_tods => 2, client_interds => 3, client_established => 4);
my $max_manuf_score = 8;

#
# Static vars
#
my $true = 1;
my $false = 0;
my $timeout = 5;

#
# Constructor
#
sub new
{
	# Standard prologue
	my $type = shift;
	my $self = {};
	bless $self, $type;

	# Constructor arguments
	$self->{'gKismetApplication'} = shift;
	$self->{'host'} = shift;
	$self->{'port'} = shift;

	# Connect to server
	if(!$self->connectServer())
	{
		return(undef);
	}

	# Enable protocols
	$self->enableProtocols();

	# Read manufacturers data
	my ($mydir) = ($0 =~ /^(.*)\//);
	$self->{'apManuf'} = $self->readManufData($mydir . '/ap_manuf');
	$self->{'clManuf'} = $self->readManufData($mydir . '/client_manuf');

	return($self);
}

#
# Destructor
#
sub DESTROY
{
	my $self = shift;
	if($self->{'socket'})
	{
		$self->{'socket'}->close();
	}
}

#
# Get socket
#
sub getSocket
{
	my $self = shift;
	return $self->{'socket'};
}

#
# Notify observers of a status change
#
sub notifyObservers
{
	my $self = shift;
	my $data = shift;

	for my $observer (@{$self->{'observers'}})
	{
		$observer->update($self, $data);
	}
}

#
# Register an observer
#
sub addObserver
{
	my $self = shift;
	my $observer = shift;
	push @{$self->{'observers'}}, $observer;
}


# Connect to server
sub connectServer
{
	my $self = shift;

	$self->{'socket'} = IO::Socket::INET->new("PeerAddr" => $self->{'host'}, "PeerPort" => $self->{'port'}, "Proto" => "tcp", "Timeout" => $timeout);
	unless($self->{'socket'})
	{
		return(0);
	}

	# Is it a kismet server we connected to?
	unless($self->{'socket'}->getline() =~ /\*KISMET/)
	{
		return(0);
	}
	return(1);
}

#
# Enable default set of protocols to watch
#
sub enableProtocols
{
	my $self = shift;
	my @protocols = qw(GPS INFO NETWORK CLIENT REMOVE ALERT STATUS);
	for my $p (@protocols)
	{
		$self->{'socket'}->print("!0 ENABLE $p *\n");
	}
}

#
# Eanble strings
#
sub enableStrings
{
	my $self = shift;
	$self->{'socket'}->print("!0 ENABLE STRINGS\n");
}

#
# Disable strings
#
sub disableStrings
{
	my $self = shift;
	$self->{'socket'}->print("!0 DISABLE STRINGS\n");
}

#
# Enable packets
#
sub enablePackets
{
	my $self = shift;
	$self->{'socket'}->print("!0 ENABLE PACKET\n");
}

#
# Disable packets
#
sub disablePackets
{
	my $self = shift;
	$self->{'socket'}->print("!0 DISABLE PACKET\n");
}

#
# Ask server to print capabilities
#
sub askCaps
{
	my $self = shift;
	my @protocols = qw(GPS INFO NETWORK CLIENT REMOVE ALERT STATUS STRINGS PACKET);
	for my $p (@protocols)
	{
		$self->{'socket'}->print("!0 CAPABILITY $p\n");
	}
}

#
# Read and parse input
#
sub readServer 
{
	my $self = shift;

	my $res = $self->{'socket'}->getline();
	if(!$res)
	{
		$self->{'gKismetApplication'}->disconnectedNotify($self);
		return;
	}
	chomp($res);
	my($cmd, $data) = ($res =~ /\*([^:]+):\s+(.*)/);

	if($cmd eq 'GPS')
	{
		my @field_names = qw(lat lon alt spd fix);
		my @fields = oneSplit($data);
		for(my $i = 0; $i <= $#fields; $i++)
		{
			$self->{'gps'}{$field_names[$i]} = $fields[$i];
		}
		$self->notifyObservers({'changed' => 'gps'});
	}
	elsif($cmd eq 'INFO')
	{
		my @field_names = qw(networks packets crypt weak noise dropped rate signal);
		my @fields = oneSplit($data);
		for(my $i = 0; $i <= $#fields; $i++)
		{
			if($field_names[$i])
			{
				$self->{'info'}{$field_names[$i]} = $fields[$i];
			}
		}
		$self->notifyObservers({'changed' => 'info'});
	}
	elsif($cmd eq 'NETWORK')
	{
		my @field_names = qw(bssid type ssid beaconinfo llcpackets datapackets cryptpackets weakpackets channel wep firsttime lasttime atype
					rangeip gpsfixed minlat minlon minalt minspd maxlat maxlon maxalt maxspd octets cloaked beaconrate maxrate
					manufkey manufscore quality signal noise bestquality bestsignal bestnoise bestlat bestlon bestalt agglat agglon
					aggalt aggpoints datasize turbocellnid turbocellmode turbocellsat carrierset);
		my @fields = oneSplit($data);
		my $bssid = $fields[0];
		for(my $i = 0; $i <= $#fields; $i++)
		{
			$self->{'network'}{$bssid}{$field_names[$i]} = $fields[$i];
		}
		$self->notifyObservers({'changed' => 'network', 'bssid' => $bssid});
	}
	elsif($cmd eq 'CLIENT')
	{
		my @field_names = qw(bssid mac type firsttime lasttime manufkey manufscore datapackets cryptpackets weakpackets gpsfixed minlat minlon
					minalt minspd maxlat maxlon maxalt maxspd agglat agglon aggalt aggpoints maxrate quality signal noise bestquality
					bestsignal bestnoise bestlat bestlon bestalt atype ip datasize);
		my @fields = oneSplit($data);
		my($bssid, $mac) = @fields[0,1];
		for(my $i = 0; $i <= $#fields; $i++)
		{
			$self->{'network'}{$bssid}{'client'}{$mac}{$field_names[$i]} = $fields[$i];
		}
		$self->notifyObservers({'changed' => 'client', 'bssid' => $bssid, 'mac' => $mac});
	}
	elsif($cmd eq 'REMOVE')
	{
		my($bssid) = oneSplit($data);
		delete $self->{'network'}{$bssid};
		$self->notifyObservers({'changed' => 'remove', 'bssid' => $bssid});
	}
	elsif($cmd eq 'ALERT')
	{
		my @field_names = qw(sec usec text);
		my @fields = oneSplit($data);
		my %a;
		for(my $i = 0; $i <= $#fields; $i++)
		{
			$a{$field_names[$i]} = $fields[$i];

		}
		push @{$self->{'alert'}}, \%a;
		$self->notifyObservers({'changed' => 'alert'});
	}
	elsif($cmd eq 'STATUS')
	{
		$self->{'status'} = $data;
		$self->notifyObservers({'changed' => 'status'});
	}
	elsif($cmd eq 'TIME')
	{
		$self->{'time'} = oneSplit($data);
		$self->notifyObservers({'changed' => 'time'});
	}
	elsif($cmd eq 'PACKET')
	{
		my @field_names = qw(type subtype timesec encrypted weak beaconrate sourcemac destmac bssid ssid prototype sourceip destip sourceport
					destport nbtype nbsource);
		my @fields = oneSplit($data);
		my($bssid, $mac) = @fields[8,6];
		my %p;
		for(my $i = 0; $i <= $#fields; $i++)
		{
			$p{$field_names[$i]} = $fields[$i];
		}
		push @{$self->{'network'}{$bssid}{'client'}{$mac}{'packets'}}, \%p;
		$self->notifyObservers({'changed' => 'packet', 'bssid' => $bssid, 'mac' => $mac});
	}
	elsif($cmd eq 'STRING')
	{
		my @field_names = qw(bssid sourcemac $text);
		my @fields = oneSplit($data);
		my($bssid, $mac) = @fields[0,1];
		my %s;
		for(my $i = 0; $i <= $#fields; $i++)
		{
			$s{$field_names[$i]} = $fields[$i];
		}
		push @{$self->{'network'}{$bssid}{'client'}{$mac}{'strings'}}, \%s;
		$self->notifyObservers({'changed' => 'string', 'bssid' => $bssid, 'mac' => $mac});
	}
}

#
# Read manufacturers data from files
#
sub readManufData
{
	my $self = shift;
	my $file = shift;
	my %manuf;
	my @field_names = qw(mac name model ssid chan ip);

	open(FILE, $file) or die "Can't open $file: $!\n";
	while(<FILE>)
	{
		my @fields = split '\t', $_;
		my $mac = $fields[0]; 
		$mac = substr($mac . ':00:00:00:00:00:00', 0, 17);
		for (my $i = 0; $i <= $#fields; $i++)
		{
			$manuf{$mac}{$field_names[$i]} = $fields[$i];
		}
	}
	close(FILE);
	return(\%manuf);
}

#
# Return printable network data
#
sub getNetworkFieldValue
{
	my $self = shift;
	my $field = shift;
	my $bssid = shift;
	my %network = %{$self->{'network'}};

	if($field eq 'SSID')
	{
		return $network{$bssid}{'ssid'};
	}
	if($field eq 'BSSID')
	{
		return $network{$bssid}{'bssid'};
	}
	elsif($field eq 'Carrier')
	{
		if($network{$bssid}{'carrierset'} & (1 << $carrier_type{'carrier_80211b'}))
		{
			return 'IEEE 802.11b';
		}
		elsif($network{$bssid}{'carrierset'} & (1 << $carrier_type{'carrier_80211a'}))
		{
			return 'IEEE 802.11a';
		}
		elsif($network{$bssid}{'carrierset'} & (1 << $carrier_type{'carrier_80211g'}))
		{
			return 'IEEE 802.11g';
		}
		elsif($network{$bssid}{'carrierset'} & (1 << $carrier_type{'carrier_80211'}))
		{
			return 'IEEE 802.11';
		}
	}
	elsif($field eq 'Manufacturer')
	{
		my $m;
		if($network{$bssid}{'type'} == $network_type{'network_adhoc'} || $network{$bssid}{'type'} == $network_type{'network_probe'})
		{
			$m = $self->{'clManuf'};
		}
		else
		{
			$m = $self->{'apManuf'};
		}

		if(defined $m->{$network{$bssid}{'manufkey'}})
		{
			return $m->{$network{$bssid}{'manufkey'}}{'name'}
		}
		else
		{
			return('Unknown');
		}
	}
	elsif($field eq 'Model')
	{
		my $m;
		if($network{$bssid}{'type'} == $network_type{'network_adhoc'} || $network{$bssid}{'type'} == $network_type{'network_probe'})
		{
			$m = $self->{'clManuf'};
		}
		else
		{
			$m = $self->{'apManuf'};
		}

		if(defined $m->{$network{$bssid}{'manufkey'}})
		{
			return $m->{$network{$bssid}{'manufkey'}}{'model'}
		}
		else
		{
			return('Unknown');
		}
	}
	elsif($field eq 'Max rate')
	{
		return $network{$bssid}{'maxrate'};
	}
	elsif($field eq 'First seen')
	{
		return scalar localtime($network{$bssid}{'firsttime'});
	}
	elsif($field eq 'Last seen')
	{
		return scalar localtime($network{$bssid}{'lasttime'});
	}
	elsif($field eq 'Clients')
	{
		my @clients = keys %{$network{$bssid}{'client'}};
		return $#{clients} + 1;
	}
	elsif($field eq 'Type')
	{
		if($network{$bssid}{'type'} == $network_type{'network_ap'})
                {
                        return 'Access Point (infrastructure)';
                }
                elsif($network{$bssid}{'type'} == $network_type{'network_probe'})
                {
                        return 'Probe request (searching client)';
                }
                elsif($network{$bssid}{'type'} == $network_type{'network_turbocell'})
                {
                        return 'Turbocell';
                }
                elsif($network{$bssid}{'type'} == $network_type{'network_data'})
                {
                        return 'Data (no network control traffic)';
                }
                elsif($network{$bssid}{'type'} == $network_type{'network_remove'})
                {
                        return 'Network remove request';
                }
	}
	elsif($field eq 'Channel')
	{
		return $network{$bssid}{'channel'};
	}
	elsif($field eq 'WEP')
	{
		$network{$bssid}{'wep'} ? return 'Yes' : return 'No';
	}
	elsif($field eq 'Beacon')
	{
		return $network{$bssid}{'beaconrate'};
	}
	elsif($field eq 'Data packets')
	{
		return $network{$bssid}{'datapackets'};
	}
	elsif($field eq 'LLC packets')
	{
		return $network{$bssid}{'llcpackets'};
	}
	elsif($field eq 'Crypt packets')
	{
		return $network{$bssid}{'cryptpackets'};
	}
	elsif($field eq 'Weak packets')
	{
		return $network{$bssid}{'weakpackets'};
	}
	elsif($field eq 'Data')
	{
		return $network{$bssid}{'datasize'} . " bytes";
	}
	elsif($field eq 'Quality')
	{
		return $network{$bssid}{'quality'} . "/" . $network{$bssid}{'bestquality'};
	}
	elsif($field eq 'Power')
	{
		return $network{$bssid}{'signal'} . "/" . $network{$bssid}{'bestsignal'};
	}
	elsif($field eq 'Noise')
	{
		return $network{$bssid}{'noise'} . "/" . $network{$bssid}{'bestnoise'};
	}
	elsif($field eq 'IP Range')
	{
		return $network{$bssid}{'rangeip'};
	}
	elsif($field eq 'IP Type')
	{
		if($network{$bssid}{'atype'} == $address_type{'address_none'})
                {
                        return 'None detected';
                }
                elsif($network{$bssid}{'atype'} == $address_type{'address_factory'})
                {
                        return 'Factory default';
                }
                elsif($network{$bssid}{'atype'} == $address_type{'address_udp'})
                {
                        return 'UDP (' .  $network{$bssid}{'octets'} . " octets)";
                }
                elsif($network{$bssid}{'atype'} == $address_type{'address_arp'})
                {
                        return 'ARP (' .  $network{$bssid}{'octets'} . " octets)";
                }
                elsif($network{$bssid}{'atype'} == $address_type{'address_tcp'})
                {
                        return 'TCP (' .  $network{$bssid}{'octets'} . " octets)";
                }
                elsif($network{$bssid}{'atype'} == $address_type{'address_dhcp'})
                {
                        return 'DHCP';
                }
                elsif($network{$bssid}{'atype'} == $address_type{'address_group'})
                {
                        return 'Group (aggregate)';
                }
                else
                {
                        return 'UNKNOWN';
                }
	}
	else
	{
		return('Not implemented');
	}
}

#
# Return printable client data
#
sub getClientFieldValue
{
	my $self = shift;
	my $field = shift;
	my $bssid = shift;
	my $mac = shift;
	my %network = %{$self->{'network'}};

	if($field eq 'MAC')
	{
		return $network{$bssid}{'client'}{$mac}{'mac'};
	}
	elsif($field eq 'Type')
	{
		if($network{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_unknown'})
		{
			return 'Unknown';
		}
		elsif ($network{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_fromds'})
		{
			return 'From Distribution (AP->Wireless)';
		}
		elsif ($network{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_tods'})
		{
			return 'To Distribution (Wireless->AP)';
		}
		elsif ($network{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_interds'})
		{
			return 'Intra-distribution (AP->AP)';
		}
		elsif ($network{$bssid}{'client'}{$mac}{'type'} == $client_type{'client_established'})
		{
			return 'Established (Wireless->AP and AP->Wireless)';
		}
		else
		{
			return 'Unknown';
		}
	}
	elsif($field eq 'Manufacturer')
	{
		if(defined $self->{'clManuf'}{$network{$bssid}{'client'}{$mac}{'manufkey'}})
		{
			return $self->{'clManuf'}{$network{$bssid}{'client'}{$mac}{'manufkey'}}{'name'};
		}
		else
		{
			return('Unknown');
		}
	}
	elsif($field eq 'Model')
	{
		if(defined $self->{'clManuf'}{$network{$bssid}{'client'}{$mac}{'manufkey'}})
		{
			return $self->{'clManuf'}{$network{$bssid}{'client'}{$mac}{'manufkey'}}{'model'}
		}
		else
		{
			return('Unknown');
		}
	}
	elsif($field eq 'First seen')
	{
		return scalar localtime $network{$bssid}{'client'}{$mac}{'firsttime'};
	}
	elsif($field eq 'Last seen')
	{
		return scalar localtime $network{$bssid}{'client'}{$mac}{'lasttime'};
	}
	elsif($field eq 'Max rate')
	{
		return $network{$bssid}{'client'}{$mac}{'maxrate'};
	}
	elsif($field eq 'WEP')
	{
		$network{$bssid}{'client'}{$mac}{'wep'} ? return 'Yes' : return 'No';
	}
	elsif($field eq 'IP')
	{
		$network{$bssid}{'client'}{$mac}{'ip'};
	}
	elsif($field eq 'Data packets')
	{
		$network{$bssid}{'client'}{$mac}{'datapackets'};
	}
	elsif($field eq 'Crypt packets')
	{
		$network{$bssid}{'client'}{$mac}{'cryptpackets'};
	}
	elsif($field eq 'Weak packets')
	{
		$network{$bssid}{'client'}{$mac}{'weakpackets'};
	}
	elsif($field eq 'Data')
	{
		$network{$bssid}{'client'}{$mac}{'datasize'} . " bytes";
	}
	elsif($field eq 'Quality')
	{
		return $network{$bssid}{'client'}{$mac}{'quality'} . "/" . $network{$bssid}{'client'}{$mac}{'bestquality'};
	}
	elsif($field eq 'Power')
	{
		return $network{$bssid}{'client'}{$mac}{'signal'} . "/" . $network{$bssid}{'client'}{$mac}{'bestsignal'};
	}
	elsif($field eq 'Noise')
	{
		return $network{$bssid}{'client'}{$mac}{'noise'} . "/" . $network{$bssid}{'client'}{$mac}{'bestnoise'};
	}
	else
	{
		return('Not iMplemented');
	}
}

#
# Return color for a network
#
sub getNetworkColor
{
	my $self = shift;
	my $bssid = shift;
	my %network = %{$self->{'network'}};

	if($network{$bssid}{'wep'} == 0)
	{
		if($network{$bssid}{'manufscore'} == $max_manuf_score)
		{
			return('Red');
		}
		return ('Yellow');
	}
	elsif($network{$bssid}{'wep'} == 1)
	{
		return ('Green');
	}
	else
	{
		return('Black');
	}
}

#
# Get info info
#
sub getInfo
{
	my $self = shift;
	my $label = shift;
	my %trInfo = ('Networks' => 'networks', 'Packets' => 'packets', 'Crypt' => 'crypt', 'Weak' => 'weak',
				'Noise' => 'noise', 'Dropped' => 'dropped', 'Rate' => 'rate', 'Signal' => 'signal');
	return $self->{'info'}{$trInfo{$label}};
}

#
# Get port
#
sub port
{
	my $self = shift;
	return $self->{'port'};
}

#
# Get host
#
sub host
{
	my $self = shift;
	return $self->{'host'};
}

#
# Get status  info
#
sub getStatus
{
	my $self = shift;
	return $self->{'status'};
}

#
# Split on space. \001 works as quotation symbol. Based on shellwords.pl
#
sub oneSplit
{
	($_) = (@_);
	my(@words, $snippet, $field);

	s/^\s+//;
	while ($_ ne '')
	{
		$field = '';
		while(1)
		{
			if(s/^\001([^\001]*)\001//)
			{
				$snippet = $1;
			}
			elsif(s/^([^\s\001]+)//)
			{
				$snippet = $1;
			}
			else
			{
				s/^\s+//;
				last;
			}
			$field .= $snippet;
		}
		push(@words, $field);
	}
	return(@words);
}

1;
