#!/usr/bin/perl -w
#
# $Id: GKismetApplication.pm,v 1.21 2003/06/11 07:04:41 solovam Exp $
#
# GKismetApplication class
#
package GKismetApplication;

use Gnome;
use strict;
use InfoBox;
use NetworkTree;
use StatusBar;
use Connection;
use ConnectionDialog;
use PrefsDialog;
use AboutDialog;
use Preferences;
use Misc;

#
#
# Constructor
#
sub new
{
	my $type = shift;
	my $self = {};
	bless $self, $type;

	# Gnome init
	init Gnome 'gkismet';

	# Create app
	my $application = new Gnome::App("gkismet", "gkismet");
	$self->{'application'} = $application;
	$application->set_default_size(Gtk::Gdk->screen_width() * 0.6, Gtk::Gdk->screen_height() * 0.8);
	$application->signal_connect("delete_event", sub {Gtk->exit(0)});

	# Preferences
	my $preferences = new Preferences();
	$self->{'preferences'} = $preferences;
	
	# Status line
	my $statusBar = new StatusBar();
	$self->{'statusBar'} = $statusBar;

	# My info box
	my $infoBox = new InfoBox();
	$self->{'infoBox'} = $infoBox;

	# Ntree
	my $networkTree = new NetworkTree($self);
	$self->{'networkTree'} = $networkTree;

	# Build menu info
	my @menuInfo =
	(
	{
		type => 'subtree', label => '_File', subtree =>
		[
			{type => 'item', label => '_Connect', pixmap_type => 'stock', pixmap_info => 'Menu_Open', callback => sub {$self->connect()}},
			{type => 'item', label => '_Disconnect', pixmap_type => 'stock', pixmap_info => 'Menu_Close', callback => sub {$self->disconnect()}},
			{type => 'separator'},
			{type => 'item', label => '_Quit', pixmap_type => 'stock', pixmap_info => 'Menu_Quit', callback => sub {Gtk->exit(0)}}
		]
	},
	{
		type => 'subtree', label => '_Settings', subtree =>
		[
			{type => 'item', label => '_Preferences', pixmap_type => 'stock', pixmap_info => 'Menu_Preferences', callback => sub{$self->preferences()}}
		]
	},
	{
		type => 'subtree', label => '_Help', subtree =>
		[
			{type => 'item', label => '_About', pixmap_type => 'stock', pixmap_info => 'Menu_About', callback => sub{$self->about()}}
		]
	}
	);

	# Build toolbar info
	my @toolbarInfo = (
		{type => 'item', label => 'Connect', pixmap_type => 'stock', pixmap_info => 'Open', 'hint' => 'Connect to Kismet server',  callback => sub {$self->connect()}},
		{type => 'item', label => 'Disconnect', pixmap_type => 'stock', pixmap_info => 'Close', 'hint' => 'Disconnect from Kismet server', callback => sub {$self->disconnect()}},
		{type => 'item', label => 'Expand', pixmap_type => 'stock', pixmap_info => 'Book Open', 'hint' => 'Expand Network Tree', callback => sub {$self->{'networkTree'}->expand()}},
		{type => 'item', label => 'Collapse', pixmap_type => 'stock', pixmap_info => 'Book Yellow', 'hint' => 'Collapse Network Tree',  callback => sub {$self->{'networkTree'}->collapse()}},
	);

	# Pack wisgets
	my $scrolledWindow = new Gtk::ScrolledWindow(undef, undef);
	$scrolledWindow->set_policy('automatic', 'automatic');
	my $vbox = new Gtk::VBox($false, 0);
	$vbox->pack_start($infoBox->getWidget(), $false, $false, 10);
	my $hbox = new Gtk::HBox($false, 0);
	$hbox->pack_start($scrolledWindow, $true, $true, 0);
	$hbox->pack_end($vbox, $false, $false, 10);
	$scrolledWindow->add($networkTree->getWidget());

	# Assemble main window
	$application->create_menus(@menuInfo);
	$application->create_toolbar(@toolbarInfo);
	$application->set_statusbar($statusBar->getWidget());
	$application->set_contents($hbox);
	$application->show_all();

	return $self;
}

#
# Connect to server
#
sub connect
{
	my $self = shift;
	my $host = shift;
	my $port = shift;

	if(defined $self->{'connection'})
	{
		$self->disconnect();
	}

	if(!defined $host || !defined $port)
	{
		my $connectionDialog = new ConnectionDialog($self);
		($host, $port) = $connectionDialog->runAndClose();
		if(!defined $host)
		{
			return(0);
		}
	}

	my $connection = new Connection($self, $host, $port);
	if(!defined $connection)
	{
		my $messagebox = new Gnome::MessageBox("Can't connect to kismet server at $host:$port", 'error', 'Button_Ok');
		$messagebox->run_and_close();
		return(0);
	}
	$self->{'connection'} = $connection;

	my $inputId = Gtk::Gdk->input_add($connection->getSocket()->fileno, 'read', sub {$connection->readServer()}, $connection->getSocket());
	$self->{'inputId'} = $inputId;

	for my $observer ($self->{'networkTree'}, $self->{'infoBox'}, $self->{'statusBar'})
	{
		$connection->addObserver($observer);
	}
	return(1);
}

#
# Disconnect
#
sub disconnect
{
	my $self = shift;

	if(!defined $self->{'connection'})
	{
		return;
	}

	Gtk::Gdk->input_remove($self->{'inputId'});

	delete $self->{'connection'};

	for my $observer ($self->{'networkTree'}, $self->{'infoBox'}, $self->{'statusBar'})
	{
		$observer->flush();
	}
}

#
# Handle server quit
#
sub disconnectedNotify
{
	my $self = shift;
	my $connection = shift;

	# The right thing to do
	$self->disconnect();
	# A nice thing to do 
	# Gtk::Gdk->input_remove($self->{'inputId'});

	my $messagebox = new Gnome::MessageBox("Lost connection to kismet server at " . $connection->host() . ":" . $connection->port(), 'error', 'Button_Ok');
	$messagebox->run_and_close();
}

#
# Run the app
#
sub run
{
	my $self = shift;

	my ($progname) = ($0 =~ /([^\/]*)$/);
	my $usage = "$progname <host> [port]\n";

	if($#ARGV > 1)
	{
		print $usage;
		exit 1;
	}

	my $host = $ARGV[0];
	my $port = $ARGV[1] ? $ARGV[1] : 2501;

	$self->connect($host, $port);

	main Gtk;

	exit(0);
}

#
# Get widget
#
sub getWidget
{
	my $self = shift;
	return $self->{'application'};
}

#
# Show properties dialog
#
sub preferences
{
	my $self = shift;

	if(!defined $self->{'preferencesDialog'})
	{
		my $preferencesDialog = new PrefsDialog($self);
		$self->{'preferencesDialog'} = $preferencesDialog;
	}
}

#
# Close preferences dialog
#
sub preferencesClose
{
	my $self = shift;
	delete $self->{'preferencesDialog'};
}

#
# Prefs have been updated
#
sub preferencesUpdateNotify
{
	my $self = shift;
	# If preferences are updated, just re-connect. Too much hassle updating object on the fly
	if(defined $self->{'connection'})
	{
		$self->connect($self->{'connection'}->host(), $self->{'connection'}->port());
	}
}

#
# Show about dialog
#
sub about
{
	my $self = shift;

	if(!defined $self->{'aboutDialog'})
	{
		my $aboutDialog = new AboutDialog($self);
		$self->{'aboutDialog'} = $aboutDialog;
	}
}

#
# Close about dialog
#
sub aboutClose
{
	my $self = shift;
	delete $self->{'aboutDialog'};
}

1;
